@@ -15,8 +15,9 @@ running them on source repos makes no sense.
 
 import errno
 import os
-from functools import partial
+import sys
 import typing
+from functools import partial
 
 from snakeoil.cli import arghparse
 from snakeoil.formatters import decorate_forced_wrapping
@@ -66,7 +67,7 @@ class DataSourceRestriction(values.base):
 
 dep_attrs = ["bdepend", "depend", "rdepend", "pdepend", "idepend"]
 metadata_attrs = dep_attrs
-dep_attrs += list(f"raw_{x}" for x in dep_attrs)
+dep_attrs += [f"raw_{x}" for x in dep_attrs]
 dep_formatted_attrs = dep_attrs + ["restrict"]
 dep_formatted_attrs = frozenset(dep_attrs + ["restrict"])
 dep_attrs = tuple(sorted(dep_attrs))
@@ -117,7 +118,7 @@ printable_attrs += (
 printable_attrs = tuple(sorted(set(printable_attrs)))
 
 
-def stringify_attr(config, pkg, attr):
+def stringify_attr(config, pkg, attr: str):
     """Grab a package attr and convert it to a string."""
     # config is currently unused but may affect display in the future.
     if attr in ("files", "uris"):
@@ -169,6 +170,18 @@ def stringify_attr(config, pkg, attr):
     return str(value)
 
 
+class PrintAttrDict(dict):
+    def __init__(self, pkg, config):
+        super().__init__()
+        self.__pkg = pkg
+        self.__config = config
+
+    def __missing__(self, key):
+        if key not in printable_attrs:
+            argparser.error(f"unknown attribute: {key}")
+        return stringify_attr(self.__config, self.__pkg, key)
+
+
 def _default_formatter(out, node):
     out.write(node, autoline=False)
     return False
@@ -366,6 +379,17 @@ def print_package(options, out, err, pkg):
         out.write()
         out.later_prefix = []
         out.wrap = False
+    elif options.format:
+        if options.atom:
+            out.write("=", autoline=False)
+        if options.atom or options.cpv:
+            out.write(pkg.cpvstr, autoline=False)
+            if options.display_slot:
+                out.write(":", pkg.slot, autoline=False)
+            if options.display_repo:
+                out.write("::", pkg.repo.repo_id, autoline=False)
+            out.write("|", autoline=False)
+        out.write(options.format.format_map(PrintAttrDict(pkg, options)))
     elif options.one_attr:
         if options.atom:
             out.write("=", autoline=False)
@@ -705,9 +729,17 @@ def bind_add_query(*args, **kwds):
     type=None,
     help="extended atom matching of pkgs",
 )
-def matches_finalize(targets, namespace):
+def matches_finalize(targets: list[str], namespace):
     repos = multiplex.tree(*namespace.repos)
 
+    if "-" in targets:
+        if not sys.stdin.isatty():
+            idx = targets.index("-")
+            in_targets = [x.strip() for x in sys.stdin.readlines()]
+            targets = targets[:idx] + in_targets + targets[idx + 1 :]
+        else:
+            argparser.error("reading from stdin is only valid when piping data in")
+
     # If current working dir is in a repo, build a path restriction; otherwise
     # match everything.
     if not targets:
@@ -757,6 +789,23 @@ add_query(
     type=parserestrict.comma_separated_containment("license"),
     help="exact match on a license",
 )
+add_query(
+    "--live",
+    action="append_const",
+    const=packages.PackageRestriction("properties", values.ContainmentMatch("live")),
+    type=None,
+    help='match packages which are have PROPERTIES="live"',
+)
+add_query(
+    "--non-live",
+    action="append_const",
+    const=packages.PackageRestriction(
+        "properties", values.ContainmentMatch("live", negate=True)
+    ),
+    type=None,
+    help='match packages which are don\'t have PROPERTIES="live"',
+)
+
 
 query.add_argument(
     "--revdep",
@@ -1079,6 +1128,17 @@ output.add_argument(
     help="print what condition(s) trigger a dep",
 )
 
+output.add_argument(
+    "-F",
+    "--format",
+    metavar="format",
+    help="print this format filled with package attributes, suppresses other output",
+    docs=f"""
+        Use the given format string to print package attributes. The format
+        uses standard Python format string syntax (see :ref:`formatspec` for
+        details), with the all attributes of ``--attr``.
+    """,
+)
 output.add_argument(
     "--attr",
     action="append",
@@ -1090,7 +1150,7 @@ output.add_argument(
         Print the given attribute's value. This option can be specified
         multiple times.
 
-        Valid attributes: {', '.join(printable_attrs)}
+        Valid attributes: {', '.join(f'``{x}``' for x in printable_attrs)}
     """,
 )
 output.add_argument(
@@ -1099,9 +1159,8 @@ output.add_argument(
     dest="attr",
     metavar="attribute",
     default=[],
-    help="like --attr but accepts any string as "
-    "attribute name instead of only explicitly "
-    "supported names",
+    help="""Like --attr but accepts any string as attribute name instead
+    of only explicitly supported names.""",
 )
 one_attr_mux = output.add_mutually_exclusive_group()
 one_attr_mux.add_argument(
@@ -1109,6 +1168,10 @@ one_attr_mux.add_argument(
     choices=printable_attrs,
     metavar="attribute",
     help="print one attribute, suppresses other output",
+    docs=f"""
+        Print the given attribute's value, while suppressing all other output.
+        All attributes of ``--attr`` are accepted.
+    """,
 )
 one_attr_mux.add_argument(
     "--force-one-attr",
